*MK_SFS1.prg
*Purpose: to create a bash script which in turn
*creates an .sfs or .pet sourcedir for Puppy Linux.
*Language: Clipper Summer '87.
*Author: P.N. Courier, "Sit Heel Speak" on the Puppy Linux forum.
*Copyright 2009-2010 by P.N.Courier
*License: released under the terms of LGPL3.
*
* MK_SFS1.prg fills OLDTREE, the .dbf of before.txt
*
*
* Clipper housekeeping:
*
clear typeahead		&& clear the keyboard buffer
close all			&& close any open .dbf and .ntx files
close proc			&& close any open procedure files
clear memory		&& deallocate any existing memory variables
vga(50)			&& enable the dmvga101 library,
				&&  to enable Clipper to use a 50-line
				&&  DOS window's full area
letbright(.t.)		&& enable bright colors
set color to W/B,N/W,B,B	&& white text on blue background
@ 0,0 clear to 49,79		&& clear the DOS window
set scoreboard off	&& turn off program status bar at line 0
altd(1)				&& turn on the Clipper debugger (Alt-D)
setcancel(.t.)		&& allow script to be halted using Alt-C
set century on		&& allows Clipper to recognize the 21st Century
*
*
* Mount .dbf file oldtree:
*
select 10			&& Allocate file handle (work area) 10
use oldtree.dbf		&& Mount oldtree.dbf on the file handle
					&&  corresponding to (i.e. it's an indirect
					&&  pointer) Clipper work area 10
* Structure for database: C:oldtree.dbf
* Number of data records:       0
* Date of last update   : 11/09/1910		&& this is dBase III+
* Field  Field Name  Type       Width    Dec	&& output, not Clipper 
*     1  TEXTLINE    Character    254
*     2  IDENTITY    Character    254
*     3  IDENTITY99  Character     99
*     4  SIZE        Character      8
*     5  TIMEDATE    Character     16
*     6  LINKSOURCE  Character    254
*     7  LINKDEST    Character    254
* ** Total **                    1140

zap					&& clear oldtree.dbf of existing records

*  Inform the user of progress:

start_date=date()
start_time=time()
@ 0,0 say "Starting MK_SFS1 (fills oldtree). Time: "
@ 0,col() say start_time
@ 1,0 say "Appending from before.txt into oldtree.dbf..."
append from before.txt sdf  && fill oldtree with contents of before.txt

run flush.com			&& DOS equivalent of linux sync command

*
*
*
*
*
*
*
*
*
*
*
go top				&& Position oldtree.dbf at record # 1
					&& ( = first line of the ls listing)

currentdir = "/"				&& to start with, we are in /
*
@ 2,0 say "Record number in Oldtree (before.txt):" 	&& show progress

do while .not.eof()		&& Process oldtree.dbf til EOF encountered

@ 2,39 say recno()			&& recno() = record number. Shows progress

*
*
*   "oldtree->textline" is an image of each ls listing line in oldtree.dbf
*
*

if left(oldtree->textline,10) == space(10)	&& blank line in the ls output
skip						&& skip to next record in oldtree.dbf
loop						&& jump to the do-while line above
endif						&& = fi in bash

if left(oldtree->textline,5) == "total"	&& ignore "total" lines in ls output
skip								&& skip to next record in oldtree.dbf
loop								&& jump to the do-while line above
endif								&& = fi in bash

if left(oldtree->textline,1) == "d"  	&& subdirectory line in listing
skip							&& "left(__,__)"
								&& = leftmost(field,# of bytes)
								&& --in other words, the leftmost
								&& character of oldtree->textline
loop							&& (ignore subdirectory lines)
endif
*
*
*
*
*
*
*
*
*
*
*
*

if left(oldtree->textline,1) == "s"		&& system bus socket
skip	&& Increment the pointer for the active database file
loop	&&  to the next record, and jump back up to the do while line.
endif	&& (in other words, ignore system bus sockets)

rttextline = rtrim(oldtree->textline)		&& rtrim = remove trailing spaces
rightmost = right(rttextline,1)		&& rightmost character of rttextline

do case

case rightmost == ":"		&& listing line naming a subdirectory
currentdir = left(rttextline,len(rttextline)-1)+"/"	&& len = stringlength

* Eliminate from consideration subdirectories which are
*  unlikely to contain files from the just-installed application:

case "/boot/"$currentdir
skip
loop

case "/cdrom/"$currentdir
skip
loop

case "/dev/"$currentdir
skip
loop

case "/ftpd/"$currentdir
skip
loop

case "/initrd/"$currentdir
skip
loop

case "/lost+found/"$currentdir
skip
loop

case "/media/"$currentdir
skip
loop

* Arguably, you might want to delete or comment-out this exclusion:
case "/mnt/"$currentdir
skip
loop

case "/proc/"$currentdir
skip
loop

case "/selinux/"$currentdir
skip
loop

case "/srv/"$currentdir
skip
loop

case "/sys/"$currentdir
skip
loop

case "/tmp/"$currentdir
skip
loop

case "/usr/src/"$currentdir
skip
loop

case "/var/"$currentdir
skip
loop

case left(rttextline,1) == "l"		&& listing line for a symlink
* Strip the permissions part, which has a lot of dashes, out:
no_perms = right(rttextline,len(rttextline) - 12)
dash_is_at = at("-",no_perms)
yearstarts = dash_is_at - 4
whole_name = right(no_perms,len(no_perms) - (yearstarts+16))
arrow_at = at("->",whole_name)  && at(__,__) = starting position of __ in __
*
*
* In Clipper, "replace...with..." fills a field within 
*  (the record you are looking at, within) a .dbf file,
*  with the value of either a memory variable or another field,
*  as opposed to x = y which fills memory variable x with value y
*
*
* alltrim(__) means, strip both leading and trailing spaces.
*  --same as ltrim(rtrim(__))
*
replace oldtree->linksource with currentdir + alltrim(left(whole_name,arrow_at - 1))
*
replace oldtree->linkdest with right(whole_name,len(whole_name) - (arrow_at + 2))
whole_name = currentdir+whole_name
replace oldtree->identity with whole_name 	&& fills the field "identity"
						&&  with the value of the
						&&  memory variable "whole_name"
						&&
						&& NOTE: in Clipper, where there
						&&  exist both a memory variable
						&&  *and* a field (within a
						&&  record, within the active (now select'ed) .dbf file)
						&&  of the same name, the replace
						&&  command will choose the field
						&&  unless the memory variable
						&&  is explicitly designated by
						&&  being prefaced with "m->"

replace oldtree->identity99 with oldtree->identity	&& fills the field "identity99"
						&&  with the value of the
						&&  field "identity" to as far
						&&  out as the (shorter) length
						&&  of field "identity99" (length
						&&  of "identity" is 254 bytes,
						&&  length of "identity99" is 
						&&  99 bytes) will allow.
						&&
						&&  The purpose of
						&&  "identity99" is to serve as
						&&  an indexing key later, at less
						&&  memory cost than indexing on
						&&  "identity" would incur.
*
*
* In Clipper, "substr(__,__,__)" means, the substring of __ starting at
*  position __ and extending for __ characters
*
* It is an idiosyncracy of Clipper that it does not 
*  accept "substring(__,__,__)", rather you must use the
*  abbreviated form "substr(__,__,__)"

replace oldtree->timedate with substr(no_perms,yearstarts,16)

case left(rttextline,1) == "-"		&& it's a file line in the listing

* Strip the permissions part out:
no_perms = right(rttextline,len(rttextline) - 12)

dash_is_at = at("-",no_perms)
yearstarts = dash_is_at - 4

replace oldtree->identity with ;	&& ; is line-continuation, like \ in bash
rtrim(currentdir) + right(no_perms,len(no_perms) - (yearstarts+16))

replace oldtree->identity99 with oldtree->identity
replace oldtree->timedate with substr(no_perms,yearstarts,16)
left_part = alltrim(left(no_perms,yearstarts - 1))
rt_space_at = rat(" ",left_part)	&& rat(__,__) = "right at"
									&&  = rightmost position of
									&&  __ in __

sizex = right(left_part,len(left_part) - rt_space_at)
replace oldtree->size with sizex		&& filesize as reported by ls

otherwise						&& error check: stop script if
								&&  a listing line of unrecognized
								&&  type is encountered.
@ 4,0 say "Halted, unknown ls listing line type encountered."
@ 5,0 say rttextline
close all  && release all file handles occupied by .dbf and .ntx files
cancel     && stop execution, don't return to any higher calling script.

endcase

skip						&& increment pointer to next record 
							&&  in oldtree.dbf

enddo						&& end of main do-while loop

select 10 && oldtree	&& not really necessary, since upon exiting the above
						&&  do-while loop we are still in work area 10,
						&&  now looking at the EOF marker of oldtree.dbf.
						&& Just, good to have a reminder of where we are ;-}
						
@ 3,0 say "Deleting non-file records in oldtree... "

* Not necessary to issue "go top" to get back up to record #1 in oldtree
*  because the "delete" command, below, assumes you want to apply it to all
*  records in the .dbf file, unless you tell it otherwise:

delete for oldtree->identity99 == space(99)	&& mark non-file lines for deletion

@ 3,col() say "Packing oldtree.dbf..."
pack					&& rewrite oldtree.dbf without
						&& records which have been
						&& marked "deleted"
						&&
						&& --this shrinks oldtree.dbf
						&&  from about 600 MB to around 14 MB.

close all				&& dismount files from handles

end_date=date()
end_time=time()
time1=realtime(end_date,end_time)-realtime(start_date,start_time)
save to savemems

@ 4,0 say "Finished MK_SFS1, no errors, in "+commas(time1)+" seconds."
@ 5,79

return					&& return to calling program,
						&&  or (when, like now, there
						&&  is no calling program)
						&&  cease execution and return
						&&  to DOS.

*
*
*
*
*
*
*
*
* Ev'ry day I get con-fused...
* Get on the bus that takes me to you...

********************************************
function commas
parameter input

if type("input")=="N"
if int(input)<>input
input=ltri(str(input,15,3))
else
input=ltri(str(input,15,0))
endif
endif

if type("input")=="C"
if val(input)<1000.and.val(input)>(-999.999)
return input
endif
endif && of if type("input")=="C"

len_w_pads=len(input)
inputx=alltrim(input)
rightpart=iif("."$inputx,right(inputx,len(inputx)-(at(".",inputx)-1)),"")
leftpart=iif("."$inputx,left(inputx,at(".",inputx)-1),inputx)
lenlp=len(leftpart)

if lenlp>0
declare backward[lenlp]
countx2=lenlp
do while countx2>0
afill(backward,substr(leftpart,countx2,1),(lenlp-countx2+1),1)
countx2=countx2-1
enddo

*A backward image of leftpart is now in backward.
bwwcommas=""
countx2=1

do while countx2<=lenlp
bwwcommas=bwwcommas+backward[countx2]
if (int(countx2/3))==(countx2/3)
bwwcommas=bwwcommas+","
endif
countx2=countx2+1
enddo

*bwwcommas now contains a backward image of alltrim(the original),
* with commas.

lenbwwc=len(bwwcommas)
leftpart=""
countx2=lenbwwc

do while countx2>0
leftpart=leftpart+substr(bwwcommas,countx2,1)
countx2=countx2-1
enddo

endif && of if lenlp>0

output=leftpart+rightpart

*Now rid leading zeroes
lenop=len(output)
declare digit[len(output)]
countx2=1

do while countx2<=lenop
afill(digit,substr(output,countx2,1),countx2,1)
countx2=countx2+1
enddo
countx2=1

do while digit[countx2]$"0,"
afill(digit," ",countx2,1)
countx2=countx2+1
enddo

countx2=1
output=""

do while countx2<=lenop
output=output+digit[countx2]
countx2=countx2+1
enddo

output=alltrim(output)
if "-,"$output
output=strtran(output,"-,","-")
endif

if len(output)<len_w_pads
output=space(len_w_pads-len(output))+output
endif

return output

********************************************
function realtime
parameters the_date,the_time

orig_date=ctod("01/01/1980")
date_secs=(the_date-orig_date)*86400
hour_secs=val(substr(the_time,1,2))*3600
min_secs=val(substr(the_time,4,2))*60
secs=val(substr(the_time,7,2))

return date_secs + hour_secs + min_secs + secs
