*MK_SFS2.prg
*Purpose: to create a bash script which in turn
*creates an .sfs or .pet sourcedir for Puppy Linux.
*Language: Clipper Summer '87.
*Author: P.N. Courier, "Sit Heel Speak" on the Puppy Linux forum.
*Copyright 2009-2010 by P.N.Courier
*License: released under the terms of LGPL3.
*
* MK_SFS2.prg fills NEWTREE, the .dbf of after.txt
*  and also fills ALLDIRS, a list of all directories in the distro tree.
*
* Clipper housekeeping:
*
clear typeahead		&& clear the keyboard buffer
close all			&& close any open .dbf and .ntx files
close proc			&& close any open procedure files
clear memory		&& deallocate any existing memory variables
vga(50)			&& enable the dmvga101 library,
				&&  to enable Clipper to use a 50-line
				&&  DOS window's full area
letbright(.t.)		&& enable bright colors
set color to W/B,N/W,B,B	&& white text on blue background
* 
set scoreboard off	&& turn off program status bar at line 0
altd(1)				&& turn on the Clipper debugger (Alt-D)
setcancel(.t.)		&& allow script to be halted using Alt-C
*
*
* Mount .dbf file newtree:
*
select 10			&& allocate file handle (work area) 10
use newtree.dbf		&& Mount newtree.dbf on the file handle
					&&  corresponding to (i.e. it's an indirect
					&&  pointer) Clipper work area 10
* Structure for database: C:newtree.dbf
* Number of data records:       0
* Date of last update   : 11/09/1910			&& Clipper Summer '87 is mostly
* Field  Field Name  Type       Width    Dec	&& OK with the 21st century,
*     1  TEXTLINE    Character    254			&& where it matters...
*     2  IDENTITY    Character    254
*     3  IDENTITY99  Character     99
*     4  SIZE        Character      8
*     5  TIMEDATE    Character     16
*     6  LINKSOURCE  Character    254
*     7  LINKDEST    Character    254
* ** Total **                    1140

zap					&& clear newtree.dbf of existing records

*  Inform the user of progress:

start_date=date()
start_time=time()
@ 6,0 say "Starting MK_SFS2 (fills newtree).  Time: "
@ 6,col() say start_time
@ 7,0 say "Appending from after.txt into newtree..."
append from after.txt sdf	&& fill newtree with contents of after.txt

run flush.com			&& DOS equivalent of linux sync command

* Mount .dbf file alldirs	&& Every directory in the new tree is
*							&&  recorded in alldirs, so that, in
select 20					&&  mk_sfs5, when a symlink is
use alldirs.dbf				&&  encountered, its target can be
zap							&&  compared against the list of
							&&  directories in alldirs.dbf, in order
							&&  to detect whether the symlink's
							&&  target is a directory or a file.
index on alldirs->directory to alldirs.ntx

select 10	&& newtree
go top				&& Position newtree.dbf at record # 1
					&& ( = first line of the ls listing)

currentdir = "/"				&& to start with, we are in /

@ 8,0 say "Record number in Newtree (after.txt):" 	&& show progress

do while .not.eof()		&& Process newtree.dbf til EOF encountered

@ 8,39 say recno()			&& recno() = record number. Shows progress

*
*
*   "newtree->textline" is an image of each ls listing line in newtree.dbf
*
*

if left(newtree->textline,10) == space(10)	&& blank line in the ls output
skip						&& skip to next record in newtree
loop						&& jump to the do-while line above
endif						&& = fi in bash

if left(newtree->textline,5) == "total"	&& ignore "total" lines in ls output
skip								&& skip to next record in newtree.dbf
loop								&& jump to the do-while line above
endif								&& = fi in bash

if left(newtree->textline,1) == "d"  		&& subdirectory line in listing
*
* Record this directory in alldirs.dbf
*
* Strip the permissions part, which has a lot of dashes, out:
rttextline = rtrim(newtree->textline)		&& rtrim = remove trailing spaces
no_perms = right(rttextline,len(rttextline) - 12)
dash_is_at = at("-",no_perms)
yearstarts = dash_is_at - 4
whole_name = right(no_perms,len(no_perms) - (yearstarts+16))

select 20		&& alldirs.dbf
append blank
*  stripdot is a function defined at the end of the main section
*   it strips the leading dot "." from currentdir
replace alldirs->directory with stripdot(currentdir) + whole_name

select 10		&& newdirs.dbf
skip
loop
endif

if left(newtree->textline,1) == "s"		&& system bus socket
skip	&& Increment the pointer for the active database file
loop	&&  to the next record, and jump back up to the do while line.
endif	&& (in other words, ignore system bus sockets)

rttextline = rtrim(newtree->textline)		&& rtrim = remove trailing spaces
rightmost = right(rttextline,1)		&& rightmost character of rttextline

do case

case rightmost == ":"		&& listing line naming a subdirectory
currentdir = left(rttextline,len(rttextline)-1) + "/"	&& len = stringlength

* Eliminate from consideration subdirectories which are
*  unlikely to contain files from the just-installed application:

case "/boot/"$currentdir
skip
loop

case "/cdrom/"$currentdir
skip
loop

case "/dev/"$currentdir
skip
loop

case "/ftpd/"$currentdir
skip
loop

case "/initrd/"$currentdir
skip
loop

case "/lost+found/"$currentdir
skip
loop

case "/media/"$currentdir
skip
loop

* Arguably, you might want to delete or comment-out this exclusion:
case "/mnt/"$currentdir
skip
loop

case "/proc/"$currentdir
skip
loop

case "/selinux/"$currentdir
skip
loop

case "/srv/"$currentdir
skip
loop

case "/sys/"$currentdir
skip
loop

case "/tmp/"$currentdir
skip
loop

case "/usr/src/"$currentdir
skip
loop

case "/var/"$currentdir
skip
loop

case left(rttextline,1) == "l"		&& listing line for a symlink
* Strip the permissions part, which has a lot of dashes, out:
no_perms = right(rttextline,len(rttextline) - 12)
dash_is_at = at("-",no_perms)
yearstarts = dash_is_at - 4
whole_name = right(no_perms,len(no_perms) - (yearstarts+16))
arrow_at = at("->",whole_name)  && at(__,__) = starting position of __ in __
*
*
* In Clipper, "replace...with..." fills a field within 
*  (the record you are looking at, within) a .dbf file,
*  with the value of either a memory variable or another field,
*  as opposed to x = y which fills memory variable x with value y
*
*
* alltrim(__) means, strip both leading and trailing spaces.
*  --same as ltrim(rtrim(__))
*
replace newtree->linksource with currentdir + alltrim(left(whole_name,arrow_at - 1))
*
replace newtree->linkdest with right(whole_name,len(whole_name) - (arrow_at+2))
whole_name = currentdir+whole_name
replace newtree->identity with whole_name 	&& fills the field "identity"
						&&  with the value of the
						&&  memory variable "whole_name"
						&&
						&& NOTE: in Clipper, where there
						&&  exist both a memory variable
						&&  *and* a field (within a
						&&  record, within the active (now select'ed) .dbf file)
						&&  of the same name, the replace
						&&  command will choose the field
						&&  unless the memory variable
						&&  is explicitly designated by
						&&  being prefaced with "m->"

replace newtree->identity99 with newtree->identity	&& fills the field "identity99"
						&&  with the value of the
						&&  field "identity" to as far
						&&  out as the (shorter) length
						&&  of field "identity99" (length
						&&  of "identity" is 254 bytes,
						&&  length of "identity99" is 
						&&  99 bytes) will allow.
						&&
						&&  The purpose of
						&&  "identity99" is to serve as
						&&  an indexing key later, at less
						&&  memory cost than indexing on
						&&  "identity" would incur.
*
*
* In Clipper, "substr(__,__,__)" means, the substring of __ starting at
*  position __ and extending for __ characters
*
* It is an idiosyncracy of Clipper that it does not 
*  accept "substring(__,__,__)", rather you must use the
*  abbreviated form "substr(__,__,__)"

replace newtree->timedate with substr(no_perms,yearstarts,16)

case left(rttextline,1) == "-"		&& it's a file line in the listing

* Strip the permissions part out:
no_perms = right(rttextline,len(rttextline) - 12)

dash_is_at = at("-",no_perms)
yearstarts = dash_is_at - 4

replace newtree->identity with ;	&& ; is line-continuation, like \ in bash
rtrim(currentdir) + right(no_perms,len(no_perms) - (yearstarts+16))

replace newtree->identity99 with newtree->identity
replace newtree->timedate with substr(no_perms,yearstarts,16)
left_part = alltrim(left(no_perms,yearstarts - 1))
rt_space_at = rat(" ",left_part)	&& rat(__,__) = "right at"
									&&  = rightmost position of
									&&  __ in __

sizex = right(left_part,len(left_part) - rt_space_at)
replace newtree->size with sizex		&& filesize as reported by ls

otherwise						&& error check: stop script if
								&&  a listing line of unrecognized
								&&  type is encountered.
@ 10,0 say "Halted, unknown ls listing line type encountered."
@ 11,0 say rttextline				
close all  && release all file handles occupied by .dbf and .ntx files
cancel     && stop execution, don't return to any higher calling script.

endcase

skip						&& increment pointer to next record 
							&&  in newtree.dbf

enddo						&& end of main do-while loop

select 10 && newtree	&& not really necessary, since upon exiting the above
						&&  do-while loop we are still in work area 10,
						&&  now looking at the EOF marker of newtree.dbf.
						&& Just, good to have a reminder of where we are ;-}

@ 9,0 say "Deleting non-file records in newtree... "

* Not necessary to issue "go top" to get back up to record #1 in newtree
*  because the "delete" command, below, assumes you want to apply it to all
*  records in the .dbf file, unless you tell it otherwise:

delete for newtree->identity99 == space(99)	&& mark blank lines for deletion

@ 9,col() say "Packing newtree.dbf..."
pack					&& rewrite newtree.dbf without
						&& records which have been
						&& marked "deleted"
						&&
						&& --this shrinks newtree.dbf
						&&  from about 600 MB to around 14 MB.

close all				&& dismount files from handles

end_date=date()
end_time=time()
time2=realtime(end_date,end_time)-realtime(start_date,start_time)
restore from savemems additive
save to savemems

@ 10,0 say "Finished MK_SFS2, no errors, in "+commas(time2)+" seconds."
@ 11,79	&& for neatness, positions cursor for C:> prompt.

return					&& return to calling program,
						&&  or (when, like now, there
						&&  is no calling program)
						&&  cease execution and return
						&&  to DOS.

********************************************
function commas
parameter input

if type("input")=="N"
if int(input)<>input
input=ltri(str(input,15,3))
else
input=ltri(str(input,15,0))
endif
endif

if type("input")=="C"
if val(input)<1000.and.val(input)>(-999.999)
return input
endif
endif && of if type("input")=="C"

len_w_pads=len(input)
inputx=alltrim(input)
rightpart=iif("."$inputx,right(inputx,len(inputx)-(at(".",inputx)-1)),"")
leftpart=iif("."$inputx,left(inputx,at(".",inputx)-1),inputx)
lenlp=len(leftpart)

if lenlp>0
declare backward[lenlp]
countx2=lenlp
do while countx2>0
afill(backward,substr(leftpart,countx2,1),(lenlp-countx2+1),1)
countx2=countx2-1
enddo

*A backward image of leftpart is now in backward.
bwwcommas=""
countx2=1

do while countx2<=lenlp
bwwcommas=bwwcommas+backward[countx2]
if (int(countx2/3))==(countx2/3)
bwwcommas=bwwcommas+","
endif
countx2=countx2+1
enddo

*bwwcommas now contains a backward image of alltrim(the original),
* with commas.

lenbwwc=len(bwwcommas)
leftpart=""
countx2=lenbwwc

do while countx2>0
leftpart=leftpart+substr(bwwcommas,countx2,1)
countx2=countx2-1
enddo

endif && of if lenlp>0

output=leftpart+rightpart

*Now rid leading zeroes
lenop=len(output)
declare digit[len(output)]
countx2=1

do while countx2<=lenop
afill(digit,substr(output,countx2,1),countx2,1)
countx2=countx2+1
enddo
countx2=1

do while digit[countx2]$"0,"
afill(digit," ",countx2,1)
countx2=countx2+1
enddo

countx2=1
output=""

do while countx2<=lenop
output=output+digit[countx2]
countx2=countx2+1
enddo

output=alltrim(output)
if "-,"$output
output=strtran(output,"-,","-")
endif

if len(output)<len_w_pads
output=space(len_w_pads-len(output))+output
endif

return output

********************************************
function realtime
parameters the_date,the_time

orig_date=ctod("01/01/1980")
date_secs=(the_date-orig_date)*86400
hour_secs=val(substr(the_time,1,2))*3600
min_secs=val(substr(the_time,4,2))*60
secs=val(substr(the_time,7,2))

return date_secs + hour_secs + min_secs + secs

********************************************
function stripdot
parameter what2strip

if left(what2strip,1) == "."
return right(what2strip,len(what2strip) - 1)
else
return what2strip
endif

* Some-thing in the way...she moves...
