*MK_SFS3.prg
*Purpose: to create a bash script which in turn
*creates an .sfs or .pet sourcedir for Puppy Linux.
*Language: Clipper Summer '87.
*Author: P.N. Courier, "Sit Heel Speak" on the Puppy Linux forum.
*Copyright 2009-2010 by P.N.Courier
*License: released under the terms of LGPL3.
*
* MK_SFS3.prg fills TREEDIFF, the differences between two distro trees
*
*
* Clipper housekeeping:
*
clear typeahead		&& clear the keyboard buffer
close all			&& close any open .dbf and .ntx files
close proc			&& close any open procedure files
clear memory		&& deallocate any existing memory variables
vga(50)			&& enable the dmvga101 library,
				&&  to enable Clipper to use a 50-line
				&&  DOS window's full area
letbright(.t.)		&& enable bright colors
set color to W/B,N/W,B,B	&& white text on blue background
*
set scoreboard off	&& turn off program status bar at line 0
altd(1)				&& turn on the Clipper debugger (Alt-D)
setcancel(.t.)		&& allow script to be halted using Alt-C
*
*
* Mount .dbf files:
*
select 10			&& allocate a file handle as work area 10
use oldtree.dbf		&& Mount oldtree.dbf on the file handle
					&&  corresponding to (i.e. it's an indirect
					&&  pointer) Clipper work area 10
					
start_date=date()
start_time=time()
@ 12,0 say "Starting MK_SFS3 (fills treediff).  Time: "
@ 12,col() say start_time
@ 12,col() say ".  Indexing oldtree..."
index on oldtree->identity99 to oldtree.ntx

select 20  			&& allocate a file handle as work area 20
use newtree.dbf		&& the .dbf of after.txt

select 30
use treediff.dbf		&& differences file
* Structure for database: C:treediff.dbf
* Number of data records:       0
* Date of last update   : 11/09/1910
* Field  Field Name  Type       Width    Dec
*     1  DIFFERENCE  Character     25
*     2  NEWTD       Character     22
*     3  OLDTD       Character     22
*     4  NEWSIZE     Character     14
*     5  OLDSIZE     Character     14
*     6  IDENTITY    Character    254
*     7  IDENTITY99  Character     99
* ** Total **                     451

zap				&& clear treediff of all records (zero out)

select 20		&& newtree.dbf
go top			&& go to record # 1 in newtree (= top line
				&&  of after.txt)

@ 13,0 say "Filling treediff.dbf:"
@ 14,0 say "Record number in newtree comparing oldtree to:"

do while.not.eof()
@ 14,48 say recno()	&& recno() = record number (in newtree.dbf)

select 10  			&& oldtree.dbf
toseek=newtree->identity99
seek m->toseek		&& idiosyncracy of seek command, needs m->
				&&  --according to the manual, seek is
				&&   *supposed* to be able to take a field
				&&   from another .dbf file as its parameter,
				&&   but I have found that it is necessary
				&&   to first copy the field to a memory
				&&   variable and then seek on the memory
				&&   variable --with the memory variable
				&&   explicitly identified as being a
				&&   memory variable.  This is done by
				&&   prefacing it with "m->" which identifies
				&&   it as being a memory variable.
do case

case .not.found()		&& It's in the new distro tree but not the old
select 30			&& treediff
append blank
replace treediff->difference with "In new, not in old"
replace treediff->identity with newtree->identity
replace treediff->identity99 with treediff->identity
replace treediff->oldsize with "Old: not in"
replace treediff->oldtd with "Old: not in"
replace treediff->newsize with "New: "+newtree->size
replace treediff->newtd with "New: "+newtree->timedate

case newtree->timedate > oldtree->timedate  && Old has an older version
select 30			&& treediff
append blank
replace treediff->difference with "Old has an older version"
replace treediff->identity with newtree->identity
replace treediff->identity99 with treediff->identity
replace treediff->newsize with "New: "+newtree->size
replace treediff->oldsize with "Old: "+oldtree->size
replace treediff->newtd with "New: "+newtree->timedate
replace treediff->oldtd with "Old: "+oldtree->timedate

case newtree->timedate < oldtree->timedate  && Old has a newer version
select 30			&& treediff
append blank
replace treediff->difference with "Old has a newer version"
replace treediff->identity with newtree->identity
replace treediff->identity99 with treediff->identity
replace treediff->newsize with "New: "+newtree->size
replace treediff->oldsize with "Old: "+oldtree->size
replace treediff->newtd with "New: "+newtree->timedate
replace treediff->oldtd with "Old: "+oldtree->timedate

case realsize(newtree->size) > realsize(oldtree->size)
						    && Old has a smaller version
select 30			&& treediff
append blank
replace treediff->difference with "Old has a smaller version"
replace treediff->identity with newtree->identity
replace treediff->identity99 with treediff->identity
replace treediff->newsize with "New: "+newtree->size
replace treediff->oldsize with "Old: "+oldtree->size
replace treediff->newtd with "New: "+newtree->timedate
replace treediff->oldtd with "Old: "+oldtree->timedate

case realsize(newtree->size) < realsize(oldtree->size)
						    && Old has a larger version
select 30 && treediff
append blank
replace treediff->difference with "Old has a larger version"
replace treediff->identity with newtree->identity
replace treediff->identity99 with treediff->identity
replace treediff->newsize with "New: "+newtree->size
replace treediff->oldsize with "Old: "+oldtree->size
replace treediff->newtd with "New: "+newtree->timedate
replace treediff->oldtd with "Old: "+oldtree->timedate

endcase

select 20 				&& newtree
skip
enddo

end_date=date()
end_time=time()
time3=realtime(end_date,end_time)-realtime(start_date,start_time)
restore from savemems additive
save to savemems

@ 15,0 say "Finished MK_SFS3, no errors, in "+commas(time3)+" seconds."
@ 16,79	&& for neatness, positions cursor for C:> prompt.

close all
return

* Whennnn logic...and proportion...ha -aave fall-llennn slow-ly de-eeead...

********************************************
function commas
parameter input

if type("input")=="N"
if int(input)<>input
input=ltri(str(input,15,3))
else
input=ltri(str(input,15,0))
endif
endif

if type("input")=="C"
if val(input)<1000.and.val(input)>(-999.999)
return input
endif
endif && of if type("input")=="C"

len_w_pads=len(input)
inputx=alltrim(input)
rightpart=iif("."$inputx,right(inputx,len(inputx)-(at(".",inputx)-1)),"")
leftpart=iif("."$inputx,left(inputx,at(".",inputx)-1),inputx)
lenlp=len(leftpart)

if lenlp>0
declare backward[lenlp]
countx2=lenlp
do while countx2>0
afill(backward,substr(leftpart,countx2,1),(lenlp-countx2+1),1)
countx2=countx2-1
enddo

*A backward image of leftpart is now in backward.
bwwcommas=""
countx2=1

do while countx2<=lenlp
bwwcommas=bwwcommas+backward[countx2]
if (int(countx2/3))==(countx2/3)
bwwcommas=bwwcommas+","
endif
countx2=countx2+1
enddo

*bwwcommas now contains a backward image of alltrim(the original),
* with commas.

lenbwwc=len(bwwcommas)
leftpart=""
countx2=lenbwwc

do while countx2>0
leftpart=leftpart+substr(bwwcommas,countx2,1)
countx2=countx2-1
enddo

endif && of if lenlp>0

output=leftpart+rightpart

*Now rid leading zeroes
lenop=len(output)
declare digit[len(output)]
countx2=1

do while countx2<=lenop
afill(digit,substr(output,countx2,1),countx2,1)
countx2=countx2+1
enddo
countx2=1

do while digit[countx2]$"0,"
afill(digit," ",countx2,1)
countx2=countx2+1
enddo

countx2=1
output=""

do while countx2<=lenop
output=output+digit[countx2]
countx2=countx2+1
enddo

output=alltrim(output)
if "-,"$output
output=strtran(output,"-,","-")
endif

if len(output)<len_w_pads
output=space(len_w_pads-len(output))+output
endif

return output

********************************************
function realtime
parameters the_date,the_time

orig_date=ctod("01/01/1980")
date_secs=(the_date-orig_date)*86400
hour_secs=val(substr(the_time,1,2))*3600
min_secs=val(substr(the_time,4,2))*60
secs=val(substr(the_time,7,2))

return date_secs + hour_secs + min_secs + secs

********************************************
function realsize
parameter textsize

do case

case "K"$textsize
multiplier=1000
textsize=strtran(textsize,"K","")

case "M"$textsize
multiplier=1000000
textsize=strtran(textsize,"M","")

case "B"$textsize
multiplier=1000000000
textsize=strtran(textsize,"B","")

case "G"$textsize
multiplier=1000000000000
textsize=strtran(textsize,"G","")

otherwise
multiplier=1

endcase

return val(textsize)*multiplier
