*MK_SFS5.prg
*Purpose: to create a bash script which in turn
*creates an .sfs or .pet sourcedir for Puppy Linux.
*Language: Clipper Summer '87.
*Author: P.N. Courier, "Sit Heel Speak" on the Puppy Linux forum.
*Copyright 2009-2010 by P.N.Courier
*License: released under the terms of LGPL3.
*
* MK_SFS5.prg writes the bash script into SCRIPT.dbf.
*  and then writes the bash script makesfs from SCRIPT.dbf.
*
* Clipper Housekeeping:
*
clear typeahead		&& clear the keyboard buffer
close all			&& close any open .dbf and .ntx files
close proc			&& close any open procedure files
clear memory		&& deallocate any open memory variables
vga(50)			&& enable the dmvga101 library,
				&&  to enable Clipper to use a 50-line
				&&  DOS window's full area
letbright(.t.)		&& enable bright colors
set color to W/B,N/W,B,B	&& white text on blue background
*
set scoreboard off	&& turn off program status bar at line 0
altd(1)				&& turn on the Clipper debugger (Alt-D)
setcancel(.t.)		&& allow script to be halted using Alt-C
*
*
*
start_date=date()
start_time=time()
@ 22,0 say "Performing MK_SFS5 (building the bash script).  Time: "+start_time

select 10
use treediff.dbf
index on difference+identity to treediff.ntx
go top
						&& Purpose of this section
						&&  is to order the lines of the
						&&  diff source (and thus the
						&&  bash script) first by nature
copy to treedif2.dbf			&&  of the difference between
use treedif2.dbf				&&  distros (e.g. this file is
erase treediff.dbf			&&  in the new but not in the
erase treediff.ntx			&&  old) and then by fullpathname,
copy to treediff.dbf			&&  so that mkdir commands in
use treediff.dbf				&&  the bash script will be
erase treedif2.dbf			&&  ordered correctly.
						&& It uses an un-indexed re-sort
						&&  of treediff to minimize
						&&  memory consumption. I found
						&&  that using an .ntx index while
						&&  running the code below, either
						&&  runs out of file handles (.ntx
						&&  files are enormously greedy
						&&  for file handles) or else
						&&  crashes Windows (probably due
						&&  to running out of heapspace).

* Optional filters

* set filter to "In new, not in old"$difference
* set filter to "Old has an older version"$difference
* set filter to "Old has a newer version""$difference
* set filter to "Old has a smaller version"$difference
* set filter to "Old has a larger version"$difference
* set filter to "In old, not in new"$difference

* For most packages, the following filter will suffice:

set filter to "In new, not in old"$difference.or.;
"Old has an older version"$difference

go top	&& In Clipper, "go top" (go to record 1, or first
		&&  record that meets the specifications of the filter)
		&&  turns on the filter.


select 20
use dirsseen				&& Directories Seen.
zap						&&
index on dirsseen->dirname to dirsseen.ntx	&& The purpose of dirsseen.dbf is
						&&  to determine that each needed
						&&  mkdir command is written to
						&&  the bash script only once.
						&& 
						&& As each filename is parsed in
						&&  stepping through treediff, its
						&&  full pathname is also parsed.
						&&  A seek command is then performed
						&&  on dirsseen.dbf, to see if 
						&&  a record of this full pathname
						&&  already exists in dirsseen.dbf.
						&&
						&& If a record of this pathname
						&&  *is not* found in dirsseen,
						&&  two things are done: 
						&&
						&& One, a mkdir command is written
						&&  into script.dbf (the source for
						&&  the bash script) so that the
						&&  bash script will create this
						&&  subdirectory in the 
						&&  build-source-directory, prior
						&&  to copying-in the files in it.
						&&
						&& And two, a record of this
						&&  pathname is created in dirsseen.
						&&
						&& If a record of this pathname
						&&  *is* found in dirsseen,
						&&  then we know that creation
						&&  of the subdirectory this file
						&&  is in has already been
						&&  accomplished by the bash script,
						&&  so it is unnecessary to write
						&&  a (duplicate) mkdir command
						&&  before inserting into script.dbf
						&&  (the bash script)
						&&  the cp command
						&&  to copy the file into
						&&  (or ln command to re-create
						&&  a symlink in) the
						&&  build-source-subdirectory.
						&&

select 30
use script.dbf
zap
add2script('#!/bin/bash')		&&  add2script is a function
								&&  defined at the end of the
								&&  main script, below.
								&&  --add2script adds each
								&&   line into the image of the
								&&   bash script

add2script('#')
add2script('# This script was automatically generated by MK_SFS1,2,3,4,5,')
add2script('# a generator for a Puppy Linux build-source-subdirectory,')
add2script('# written by P.N. Courier, a.k.a. "Sit Heel Speak" on the')
add2script('# Puppy Linux forum, September 2009 to November 2010.')
add2script('# Released under the terms of license LGPL3.')
add2script('#')
add2script('# Make sure /mnt/sdb1/sourcedir is empty before you run this script.')
add2script('#')
add2script('# This script needs to have its line endings converted')
add2script('# from CRLF to LF in Geany (Document->Set Line Endings)')
add2script('# and then made executeable, before it is run.')

select 40
use alldirs.dbf		&& list of all directories in the new tree
@ 23,0 say "Indexing alldirs.dbf (list of all directories in the new tree)."
index on alldirs->directory to alldirs.ntx

select 10		&& treediff
go top

add2script('#')
add2script('#')
add2script('#')
add2script('# '+rtrim(treediff->difference))
add2script('#')
add2script('#')
add2script('#')
lastdiff=treediff->difference

dirlastat="/"  		&& dirlastat = where we are at this point
					&& in the tree-walk

builddir="/mnt/sdb1/sourcedir/"

do while .not.eof()	&& step sequentially through treediff.dbf

@ 24,0 say "Record number in treediff: "	&& inform the user of progress
@ 24,col() say recno()

* Insert a prominent comment in the bash script to denote if nature of 
*  difference has changed,
*  i.e., if moving now from "in new but not old" files to 
*  "old has an older version" files.

if treediff->difference<>lastdiff
add2script('#')
add2script('#')
add2script('#')
add2script('# '+rtrim(treediff->difference))
add2script('#')
add2script('#')
add2script('#')
lastdiff=treediff->difference
endif

rtrimid=rtrim(treediff->identity)		&& strip trailing spaces

if "->"$rtrimid					&& It's a symlink
linkat=at("->",rtrimid)
rtrimid=left(rtrimid,linkat-1)
rtrimid=rtrim(rtrimid)
endif

* Provide for creation of the subdirectory to hold the file

rslashat=rat("/",rtrimid)		&& rat(__,__) = rightmost position
								&&  of __ in __

subdirname=left(rtrimid,rslashat-1)
lensdname=len(subdirname)		&& length in bytes of subdirname
pathsofar=""					&& scratch buildstring for pathname

indx=1
do while indx<=lensdname
thischar=substr(subdirname,indx,1)	&& substring of subdirname
						&&  starting at character # (indx)
						&&  (first character = # 1)
						&&  and extending
						&&  over a length of 1 character
						&& --in other words, the character
						&&  at position # (indx)
if thischar=="/"

if .not.(pathsofar==".")
select 20	&& dirsseen
toseek=pathsofar+space(254-len(pathsofar))
seek m->toseek		&& "m->" denotes that this variable name
					&&  refers to a memory variable and not
					&&  to the name of a field within a
					&&  record within a .dbf file.  It is
					&&  an idiosyncracy of the seek command
					&&  in Clipper Summer '87, that it needs
					&&  a memory variable as the seek target.
					&&  The manual says, seek can also accept
					&&  as its target a field within a record 
					&&  of a .dbf file, but no, seek will
					&&  often crash if you attempt to seek
					&&  for a match to a field rather than
					&&  for a match to a memory variable.
					&&
					&& Moreover, seek *needs* the memory
					&&  variable to be prefixed with m->
					&&
					&& The memory variable must be filled
					&&  out with spaces in order to exactly
					&&  match the length of the field in the
					&&  indexed .dbf file on which the seek
					&&  command is invoked.
					&&
	
if .not.found()
append blank
replace dirsseen->dirname with pathsofar
add2script('mkdir '+'"'+builddir+pathsofar+'"')
endif  && of if this subdir was .not.found()
endif  && of if .not.(pathsofar=".")
endif  && of if thischar=="/"

pathsofar=pathsofar+thischar
indx=indx+1
enddo

*The above do-while loop identifies all subdirectories above the one
* this file is in.
* Now check whether the subdirectory this file is in is
* already recorded in dirsseen (and its mkdir line is in script.dbf):

toseek=subdirname+space(254-lensdname)
select 20 && dirsseen  --not really necessary, we're still in work area 20
seek m->toseek

if .not.found()	&& if this subdir name is not already in dirsseen
append blank
replace dirsseen->dirname with subdirname

add2script('mkdir '+'"'+builddir+subdirname+'"')
endif  && of if this subdirectory was .not.found() in dirsseen

if .not.(dirlastat==builddir+subdirname)
add2script("cd "+'"'+builddir+subdirname+'"')
dirlastat=builddir+subdirname
endif

* Do not write to the bash script, a cp line for any symbolic link.
* The cp command has problems with symbolic links.
* Instead, symbolic links are explicitly re-created:

if .not.("->"$treediff->identity)		&& not a symbolic link
add2script('cp '+'"'+rtrimid+'"'+' .')
else							&& it is a symbolic link

*Diagnostic, while debugging the case where a symlink
* points to a subdirectory

@ 28,0 say "subdirname:"
@ 29,0 say subdirname

add2script("#")
add2script("# Symbolic link in this subdirectory:")
add2script("#")

* Extract link name
arrow_at=rat("->",treediff->identity)
left_part=left(treediff->identity,arrow_at-1)
left_part=alltrim(left_part)
ln_slashat=rat("/",left_part)
link_name=right(left_part,len(left_part)-ln_slashat)

@ 30,0 say "link_name:"
@ 31,0 say link_name

* Extract link target
target=right(treediff->identity,len(treediff->identity)-(arrow_at+1))
target=alltrim(target)

@ 32,0 say "target:"
@ 33,0 say target

* Detect whether this symlink is to a file or to a subdirectory

* I am not too sure of the following hack.  It is an attempt
*  to identify the real target of the symlink
sdlinkisin=stripdot(subdirname)	&& subdirectory the link is in
realtsubd=sdlinkisin			&& scratch string used below
						&& --stripped from the right
targtimage=target				&& scratch string used below
						&& --stripped from the left
do while .t.
if len(targtimage)<4.and.(.not.(targtimage=="../"))
exit
endif
* Check whether the symlink's target points to a higher directory
* If not, get out of this do while loop
targ_left3=left(targtimage,3)
if .not.(left(targ_left3,3)=="../")
exit
endif
* Target is pointing upstream one or more directories, so
* adjust realtsubd
rslatinrt=rat("/",realtsubd)
if rslatinrt>1
realtsubd=left(realtsubd,rslatinrt-1)
else
realtsubd="/"
endif
targtimage=right(targtimage,len(targtimage)-3)
enddo
realtsubd=fixslash(realtsubd+targtimage)
*** end of horrible hack


select 40		&& alldirs
toseek=realtsubd+space(254-len(realtsubd))
seek m->toseek

if found()		&& symlink target is a directory
add2script('ln -s '+'"'+target+'"'+' '+'"'+link_name+'"')

@ 34,0 say "Target is found in ALLDIRS, must be to a directory"
@ 35,0 say rtrim(alldirs->directory)

else			&& symlink target is a file
add2script('ln -s -T '+'"'+target+'"'+' '+'"'+link_name+'"')

@ 34,0 say "Not found in ALLDIRS, must be a file"

endif	&& of if/else target is found in list of directories

add2script("#")

dummyget="X"
@ 36,0 say "Press a key:" get dummyget pict "X"
clear typeahead
read
@ 28,0 clear to 36,79

endif	&& of if/else .not.("->"$treediff->identity) (is symlink or not)

select 10 		&& treediff
skip
enddo

@ 25,0 say "Writing out the bash script text..."
close all

select 30
use script.dbf
scr_recs=reccount()		&& reccount() = # of records in this .dbf file

go top

if file("makesfs")
erase makesfs
endif

set console off
set printer to makesfs.
set printer on

do while .not.eof()
script_rec=recno()

if script_rec<scr_recs
? rtrim(textline)
else
? rtrim(textline)+chr(13)+chr(10)	&& add CRLF to the last line
endif

skip
enddo

set printer off
set printer to
set console on

* Eliminate CRLF at top (an artefact of set printer on)
run filemod makesfs. /Qc /E /T\013\010#! /R#!

end_date=date()
end_time=time()
time5=realtime(end_date,end_time)-realtime(start_date,start_time)

*public time1
restore from savemems additive

num_secs=commas(time5)
@ 26,0 say "Finished MK_SFS5, no errors, in ";
+num_secs+iif(num_secs=="1"," second.",+" seconds.")

@ 28,0 say "Your bash script is makesfs."
@ 29,0 say "Edit it to eliminate unwanted inclusions in your package,"
@ 30,0 say "then change its line endings from CRLF to LF and make it executeable"
@ 31,0 say "before running makesfs."

@ 33,0 say "Total running time, mk_sfs1 through mk_sfs5: "
num_secs=commas(time1+time2+time3+time4+time5)
@ 33,col() say num_secs+" seconds."

return

****  

* "Many is the word,
*  That only leaves ya guessing,
*  Ya guess about a thing
*  You really ought to know"

*******************************************************
function add2script
parameter whattoadd

old_area=select()
select 30 && script
append blank
replace textline with fixslash(whattoadd)
select (old_area)
return 0

*******************************************************
function commas
parameter input

if type("input")=="N"
if int(input)<>input
input=ltri(str(input,15,3))
else
input=ltri(str(input,15,0))
endif
endif

if type("input")=="C"
if val(input)<1000.and.val(input)>(-999.999)
return input
endif
endif && of if type("input")=="C"

len_w_pads=len(input)
inputx=alltrim(input)
rightpart=iif("."$inputx,right(inputx,len(inputx)-(at(".",inputx)-1)),"")
leftpart=iif("."$inputx,left(inputx,at(".",inputx)-1),inputx)
lenlp=len(leftpart)

if lenlp>0
declare backward[lenlp]
countx2=lenlp
do while countx2>0
afill(backward,substr(leftpart,countx2,1),(lenlp-countx2+1),1)
countx2=countx2-1
enddo

*A backward image of leftpart is now in backward.
bwwcommas=""
countx2=1

do while countx2<=lenlp
bwwcommas=bwwcommas+backward[countx2]
if (int(countx2/3))==(countx2/3)
bwwcommas=bwwcommas+","
endif
countx2=countx2+1
enddo

*bwwcommas now contains a backward image of alltrim(the original),
* with commas.

lenbwwc=len(bwwcommas)
leftpart=""
countx2=lenbwwc

do while countx2>0
leftpart=leftpart+substr(bwwcommas,countx2,1)
countx2=countx2-1
enddo

endif && of if lenlp>0

output=leftpart+rightpart

*Now rid leading zeroes
lenop=len(output)
declare digit[len(output)]
countx2=1

do while countx2<=lenop
afill(digit,substr(output,countx2,1),countx2,1)
countx2=countx2+1
enddo
countx2=1

do while digit[countx2]$"0,"
afill(digit," ",countx2,1)
countx2=countx2+1
enddo

countx2=1
output=""

do while countx2<=lenop
output=output+digit[countx2]
countx2=countx2+1
enddo

output=alltrim(output)
if "-,"$output
output=strtran(output,"-,","-")
endif

if len(output)<len_w_pads
output=space(len_w_pads-len(output))+output
endif

return output

********************************************
function fixslash
parameter whattofix
fixed=whattofix

fixed=strtran(fixed,"/./","/")
fixed=strtran(fixed,"//","/")
fixed=strtran(fixed,'cp "./','cp "/')

if right(fixed,2)=="/."
fixed=left(rtfixed,len(fixed)-2)
endif

fixed=strtran(fixed,". .",".")

return fixed

********************************************
function realtime
parameters the_date,the_time

orig_date=ctod("01/01/1980")
date_secs=(the_date-orig_date)*86400
hour_secs=val(substr(the_time,1,2))*3600
min_secs=val(substr(the_time,4,2))*60
secs=val(substr(the_time,7,2))

return date_secs + hour_secs + min_secs + secs

********************************************
function stripdot
parameter what2strip

if left(what2strip,1) == "."
return right(what2strip,len(what2strip) - 1)
else
return what2strip
endif

